// Importing specific functions and objects from the 'd3' library.
import {
  select,
  csv,
  scaleLinear,
  max,
  scaleBand,
  axisLeft,
  axisBottom,
  format
} from 'd3';

// Selecting the SVG element in the HTML document and storing it in the 'svg' variable.
const svg = select('svg');

// Converting the width and height attributes of the SVG element to numbers.
const width = +svg.attr('width');
const height = +svg.attr('height');

// Defining the 'render' function, which takes 'data' as input and is responsible for creating the bar chart.
const render = data => {
    // Functions to extract the relevant data values for the x and y axes.
  const xValue = d => d['Number of Software Developers'];
  const yValue = d => d.State;
  
	// Defining the margins for the chart within the SVG element.
  const margin = { top: 20, right: 40, bottom: 77, left: 180 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  // Creating the xScale, which maps the data values to the horizontal position of the bars.
  const xScale = scaleLinear()
    .domain([0, 650000])
    .range([0, innerWidth]);
  
  // Creating the yScale, which maps the data values to the vertical position of the bars.
  const yScale = scaleBand()
    .domain(data.map(yValue))
    .range([0, innerHeight])
    .padding(0.2);
  
  // Appending a 'g' element to the SVG to create a container for the chart and applying margins to it.
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const xAxis = axisBottom(xScale)
  
  g.append('g').call(axisLeft(yScale))
       .style("font-size", "14px")
  
  // Creating and appending the x-axis to the 'g' element using the xScale.		
  const xAxisG = g.append('g').call(xAxis)
     .style("font-size", "14px")
    .attr('transform', `translate(0,${innerHeight})`);
  
 // Creating and appending the bars to the chart using the data and scales.
  g.selectAll('rect').data(data)
    .enter().append('rect')
      .attr('y', d => yScale(yValue(d)))
      .attr('width', d => xScale(xValue(d)))
      .attr('height', yScale.bandwidth());
  
};

// Loading the data from the 'data.csv' file and calling the 'render' function to create the chart.
csv('data.csv').then(data => {
  render(data);
});